with Ada.Text_IO,Reps,Taylors,Taylors.Ops,Taylors.IO;
use  Ada.Text_IO, Reps;

generic

  PDeg: in Positive;
  type Scalar is private;

  with function IsNumeric(S: Scalar) return Boolean is <>;
  with function Approx(S: Scalar) return Rep is <>;
  with function Scal(K: Integer) return Scalar is <>;
  with function Scal(R: Rep) return Scalar is <>;
  with function Scal(R1,R2: Rep) return Scalar is <>;
  with function Ball(R: Rep) return Scalar is <>;
  with function Ball(S: Scalar) return Scalar is <>;
  with function "<"(S1,S2: Scalar) return Boolean is <>;
  with function Inf(S: Scalar) return Rep is <>;
  with function Sup(S: Scalar) return Rep is <>;
  with function InfAbs(S: Scalar) return Rep is <>;
  with function SupAbs(S: Scalar) return Rep is <>;
  with function Norm(S: Scalar) return Scalar is <>;
  with function "abs"(S: Scalar) return Scalar is <>;
  with function Min(S1,S2: Scalar) return Scalar is <>;
  with function Max(S1,S2: Scalar) return Scalar is <>;
  with function "-"(S: Scalar) return Scalar is <>;
  with procedure Split(S: in Scalar; S0,SE: out Scalar) is <>;
  with function "+"(S,T: Scalar) return Scalar is <>;
  with function "-"(S,T: Scalar) return Scalar is <>;
  with function "*"(R: Rep; S: Scalar) return Scalar is <>;
  with function "/"(S: Scalar; R: Rep) return Scalar is <>;
  with function Sqr(S: Scalar) return Scalar is <>;
  with function "*"(S,T: Scalar) return Scalar is <>;
  with function Inv(S: Scalar) return Scalar is <>;
  with function "/"(S,T: Scalar) return Scalar is <>;
  with function "**"(S: Scalar; I: Integer) return Scalar is <>;
  with function Exp(S: Scalar) return Scalar is <>;
  with function Log(S: Scalar) return Scalar is <>;
  with function Sqrt(S: Scalar) return Scalar is <>;
  with function Widen(S: Scalar) return Scalar is <>;
  with procedure Show(S: in Scalar; NewLine: in Boolean := True) is <>;
  with procedure Get(F: in File_Type; S: out Scalar; Decimal: in Boolean := True) is <>;
  with procedure Put(F: in File_Type; S: in Scalar; Decimal: in Boolean := True) is <>;

package Basics is

  package TCoeffs is new Taylors(PDeg => PDeg, Scalar => Scalar);
  package TCoeffs_Ops is new TCoeffs.Ops;
  package TCoeffs_IO is new TCoeffs.IO ;
  use TCoeffs, TCoeffs_Ops, TCoeffs_IO;

  function FirstStep(R: Radius; Gm2: Scalar) return Taylor;
  -- computation of the first step, see subsection 4.3
  function SecondStep(R: Radius; Gm2,D0,D1,S: Scalar) return Taylor;
  -- computation of the following steps, see subsection 4.3
  function CFirst(R: Radius; Gm2: Scalar; U: Taylor) return Taylor;  -- map F_\gamma
  function CSecond(R: Radius; Gm2,D0,D1,S: Scalar; U: Taylor) return Taylor; -- map C_\gamma
  function Lip1(R: Radius; Gm2,U: Scalar) return Scalar;  -- Lipschitz constant of F_\gamma
  function Lip2(R: Radius; Gm2,S,U: Scalar) return Scalar;-- Lipschitz constant of C_\gamma
  procedure EndPointU(R0: in radius; R,Eps: in Scalar; U: in Taylor;
                     D0,D1: out Scalar);  -- W_0 and W_1
  procedure EndPointW(R0: in radius; R,Eps: in Scalar; U: in Taylor; D0,D1: out Scalar); -- V_0 and V_1
  function Rad(Gm2,S: Scalar; U: Taylor) return Radius;  -- choice of T for following steps
  function FindZero(R: Radius; DS: Scalar; U: Taylor) return Scalar;
  function Integrate(Gm2: Scalar) return Scalar;
  function Check(MinP,MaxP:Rep; File: String) return Boolean;
end Basics;
