with Ada.Text_IO, Messages, Reps.Ops, Reps.IO, Linear.IO;
use Ada.Text_IO, Messages, Reps.Ops, Reps.IO, Linear.IO;
with Intervals,Intervals.Ops,Intervals.IO;
use  Intervals,Intervals.Ops,Intervals.IO;

with Basics;

function Integrate(N,PDeg: Positive; DT,B1,B2: Rep;
                   Down,Up: Boolean) return Boolean is

   package Txt_Int_IO is new Ada.Text_IO.Integer_IO(Integer);

   subtype Scalar is Interval;

   package Basic is new Basics (PDeg => PDeg, Scalar => Scalar);
   package TC renames Basic.TCoeffs;
   package TC_Ops Renames Basic.TCoeffs_Ops;
   package TC_IO renames Basic.TCoeffs_IO;
   use Basic,TC,TC_Ops,TC_IO;

   U,V: Taylor;
   R0: Radius := 1.2;
   R1: Radius := 0.3;
   Beta: constant Scalar := Scal(B1,B2);
   MaxRepet: constant Natural := 40;
   C,R,Nm,Sp,Eps,D0,D1,D2,D3: Scalar;
   T: Rep := Zero;
begin
   C := Scal(0.37);
   R := Scal(0.45);
   if FlowOut(C,R) <= Zero then
      Show("not out");
      return False;
   end if;
   if FlowIn(C,R) >= Zero then
      Show("not in");
      return False;
   end if;
   U := FirstStep(R0,Beta,N);
   Eps := Norm(R0,U-CFirst(R0,Beta,N,U))/(Scal(One)-Lip1(R0,N));
   EndPointU(R0,Scal(One),Eps,U,D0,D1,D2,D3);

   while T < DT loop
      U := SecondStep(R1,D0,D1,D2,D3,N);
      V := U-CSecond(R1,D0,D1,D2,D3,N,U);
      SetComponent(0,Scal(Zero),V);
      SetComponent(1,Scal(Zero),V);
      SetComponent(2,Scal(Zero),V);
      SetComponent(3,Scal(Zero),V);
      Eps := Norm(R1,V);
      Eps := Eps/(Scal(One)-Lip2(R1,Sup(Eps+Norm(R1,U)),N));
      EndPointW(R1,Scal(Fourth),Eps,U,D0,D1,D2,D3);
      T := T+Fourth;
   end loop;
   EndPointW(R1,Scal(DT-T+Fourth),Eps,U,D0,D1,D2,D3);
   Nm := Norm(D0,D1,D2,D3);
   Sp := Alpha(D0,D1,D2,D3)/Nm;
   if Down and Sup(Sp) > Inf(-C) then
      Show(Sp);
      Show(C);
      Show("not down");
      return False;
   end if;
   if Up and Inf(Sp) < Sup(C) then
      Show("not up");
      return False;
   end if;
   if Sup(Nm) > Inf(R) then
      Show(Nm);
      Show(R);
      Show("not in ball");
      return False;
   end if;
   return True;

end Integrate;
